@@ -6,12 +6,14 @@ from collections.abc import Iterable, Iterator
 from dataclasses import asdict, dataclass, field
 from datetime import datetime
 from enum import Enum
+from importlib.metadata import version
 import json
 import logging
 import math
 import os
 import re
 import shutil
+import signal
 import socket
 import string
 import subprocess
@@ -19,10 +21,11 @@ import sys
 import textwrap
 import threading
 import time
+from types import FrameType
 from typing import IO, Any, Optional, TextIO
 
-__version__ = "0.11.0"
-__schema_version__ = "0.2.0"
+__version__ = version("con-duct")
+__schema_version__ = "0.2.2"
 
 
 lgr = logging.getLogger("con-duct")
@@ -66,6 +69,12 @@ Resource usage is determined by polling (at a sample-interval).
 During execution, duct produces a JSON lines (see https://jsonlines.org) file
 with one data point recorded for each report (at a report-interval).
 
+limitations:
+  Duct uses session id to track the command process and its children, so it
+  cannot handle the situation where a process creates a new session.
+  If a command spawns child processes, duct will collect data on them, but
+  duct exits as soon as the primary process exits.
+
 environment variables:
   Many duct options can be configured by environment variables (which are
   overridden by command line options).
@@ -76,6 +85,7 @@ environment variables:
   DUCT_SAMPLE_INTERVAL: see --sample-interval
   DUCT_REPORT_INTERVAL: see --report-interval
   DUCT_CAPTURE_OUTPUTS: see --capture-outputs
+  DUCT_MESSAGE: see --message
 """
 
 
@@ -121,6 +131,14 @@ class RecordTypes(str, Enum):
         return self is RecordTypes.ALL or self is RecordTypes.PROCESSES_SAMPLES
 
 
+class SessionMode(str, Enum):
+    NEW_SESSION = "new-session"
+    CURRENT_SESSION = "current-session"
+
+    def __str__(self) -> str:
+        return self.value
+
+
 @dataclass
 class SystemInfo:
     cpu_total: int
@@ -350,9 +368,11 @@ class Report:
         arguments: list[str],
         log_paths: LogPaths,
         summary_format: str,
+        working_directory: str,
         colors: bool = False,
         clobber: bool = False,
         process: subprocess.Popen | None = None,
+        message: str = "",
     ) -> None:
         self._command = command
         self.arguments = arguments
@@ -360,6 +380,7 @@ class Report:
         self.summary_format: str = summary_format
         self.clobber = clobber
         self.colors = colors
+        self.message = message
         # Defaults to be set later
         self.start_time: float | None = None
         self.process = process
@@ -373,6 +394,7 @@ class Report:
         self.end_time: float | None = None
         self.run_time_seconds: str | None = None
         self.usage_file: TextIO | None = None
+        self.working_directory: str = working_directory
 
     def __del__(self) -> None:
         safe_close_files([self.usage_file])
@@ -452,6 +474,7 @@ class Report:
             output = subprocess.check_output(
                 [
                     "ps",
+                    "-w",
                     "-s",
                     str(self.session_id),
                     "-o",
@@ -498,6 +521,7 @@ class Report:
         if self.usage_file is None:
             self.usage_file = open(self.log_paths.usage, "w")
         self.usage_file.write(json.dumps(self.current_sample.for_json()) + "\n")
+        self.usage_file.flush()  # Force flush immediately
 
     @property
     def execution_summary(self) -> dict[str, Any]:
@@ -523,6 +547,7 @@ class Report:
             "num_reports": self.number,
             "start_time": self.start_time,
             "end_time": self.end_time,
+            "working_directory": self.working_directory,
         }
 
     def dump_json(self) -> str:
@@ -538,6 +563,8 @@ class Report:
                 "schema_version": __schema_version__,
                 "execution_summary": self.execution_summary,
                 "output_paths": asdict(self.log_paths),
+                "working_directory": self.working_directory,
+                "message": self.message,
             }
         )
 
@@ -622,6 +649,20 @@ class SummaryFormatter(string.Formatter):
             return "%s%s%s" % (self.COLOR_SEQ % color, s, self.RESET_SEQ)
         return s
 
+    def _format_duration(self, value: float) -> str:
+        """Format a duration in seconds to human-readable format."""
+        if value >= 3600:  # >= 1 hour
+            hours = int(value // 3600)
+            minutes = int((value % 3600) // 60)
+            seconds = value % 60
+            return f"{hours}h {minutes}m {seconds:.1f}s"
+        elif value >= 60:  # >= 1 minute
+            minutes = int(value // 60)
+            seconds = value % 60
+            return f"{minutes}m {seconds:.1f}s"
+        else:
+            return f"{value:.2f}s"
+
     def convert_field(self, value: str | None, conversion: str | None) -> Any:
         if conversion == "S":  # Human size
             if value is not None:
@@ -641,6 +682,25 @@ class SummaryFormatter(string.Formatter):
                 return self.color_word(self.NONE, self.RED)
             else:
                 return self.color_word(value, self.GREEN)
+        elif conversion == "P":  # Percentage
+            if value is not None:
+                return f"{float(value):.2f}%"
+            else:
+                return self.color_word(self.NONE, self.RED)
+        elif conversion == "T":  # Time duration
+            if value is not None:
+                return self._format_duration(float(value))
+            else:
+                return self.color_word(self.NONE, self.RED)
+        elif conversion == "D":  # DateTime from timestamp
+            if value is not None:
+                try:
+                    dt = datetime.fromtimestamp(float(value))
+                    return dt.strftime("%b %d, %Y %I:%M %p")
+                except (ValueError, OSError):
+                    return str(value)
+            else:
+                return self.color_word(self.NONE, self.RED)
 
         return super().convert_field(value, conversion)
 
@@ -684,6 +744,8 @@ class Arguments:
     colors: bool
     log_level: str
     quiet: bool
+    session_mode: SessionMode
+    message: str = ""
 
     def __post_init__(self) -> None:
         if self.report_interval < self.sample_interval:
@@ -810,6 +872,23 @@ class Arguments:
             type=RecordTypes,
             help="Record system-summary, processes-samples, or all",
         )
+        parser.add_argument(
+            "-m",
+            "--message",
+            type=str,
+            default=os.getenv("DUCT_MESSAGE", ""),
+            help="Record a descriptive message about the purpose of this execution. "
+            "You can also provide value via DUCT_MESSAGE env variable.",
+        )
+        parser.add_argument(
+            "--mode",
+            default="new-session",
+            choices=list(SessionMode),
+            type=SessionMode,
+            help="Session mode: 'new-session' creates a new session for the command (default), "
+            "'current-session' tracks the current session instead of starting a new one. "
+            "Useful for tracking slurm jobs or other commands that should run in the current session.",
+        )
         args = parser.parse_args(
             args=cli_args,
             namespace=cli_kwargs and argparse.Namespace(**cli_kwargs) or None,
@@ -829,6 +908,8 @@ class Arguments:
             colors=args.colors,
             log_level=args.log_level,
             quiet=args.quiet,
+            session_mode=args.mode,
+            message=args.message,
         )
 
 
@@ -986,6 +1067,27 @@ def main() -> None:
     sys.exit(execute(args))
 
 
+class ProcessSignalHandler:
+    def __init__(self, pid: int) -> None:
+        self.pid: int = pid
+        self.sigcount: int = 0
+
+    def handle_signal(self, _sig: int, _frame: Optional[FrameType]) -> None:
+        self.sigcount += 1
+        if self.sigcount == 1:
+            lgr.info("Received SIGINT, passing to command")
+            os.kill(self.pid, signal.SIGINT)
+        elif self.sigcount == 2:
+            lgr.info("Received second SIGINT, again passing to command")
+            os.kill(self.pid, signal.SIGINT)
+        elif self.sigcount == 3:
+            lgr.warning("Received third SIGINT, forcefully killing command process")
+            os.kill(self.pid, signal.SIGKILL)
+        elif self.sigcount >= 4:
+            lgr.critical("Exiting duct, skipping cleanup")
+            os._exit(1)
+
+
 def execute(args: Arguments) -> int:
     """A wrapper to execute a command, monitor and log the process details.
 
@@ -1009,6 +1111,7 @@ def execute(args: Arguments) -> int:
     else:
         stderr_file = stderr
 
+    working_directory = os.getcwd()
     full_command = " ".join([str(args.command)] + args.command_args)
     files_to_close = [stdout_file, stdout, stderr_file, stderr]
 
@@ -1017,8 +1120,10 @@ def execute(args: Arguments) -> int:
         args.command_args,
         log_paths,
         args.summary_format,
+        working_directory,
         args.colors,
         args.clobber,
+        message=args.message,
     )
     files_to_close.append(report.usage_file)
 
@@ -1028,7 +1133,8 @@ def execute(args: Arguments) -> int:
             [str(args.command)] + args.command_args,
             stdout=stdout_file,
             stderr=stderr_file,
-            start_new_session=True,
+            start_new_session=(args.session_mode == SessionMode.NEW_SESSION),
+            cwd=report.working_directory,
         )
     except FileNotFoundError:
         # We failed to execute due to file not found in PATH
@@ -1040,10 +1146,19 @@ def execute(args: Arguments) -> int:
         print(f"{args.command}: command not found", file=sys.stderr)
         return 127  # seems what zsh and bash return then
 
+    handler = ProcessSignalHandler(process.pid)
+    signal.signal(signal.SIGINT, handler.handle_signal)
     lgr.info("duct is executing %r...", full_command)
     lgr.info("Log files will be written to %s", log_paths.prefix)
     try:
-        report.session_id = os.getsid(process.pid)  # Get session ID of the new process
+        if args.session_mode == SessionMode.NEW_SESSION:
+            report.session_id = os.getsid(
+                process.pid
+            )  # Get session ID of the new process
+        else:  # CURRENT_SESSION mode
+            report.session_id = os.getsid(
+                os.getpid()
+            )  # Get session ID of duct's own process
     except ProcessLookupError:  # process has already finished
         # TODO: log this at least.
         pass
